#!/bin/bash

set -eu

usage() {
    cat <<EOF
Usage: makecrx.sh --key PEMFILE --source PATH --output FILE

--key 	    Private key file
--source    Source directory
--output    Output file
EOF
    exit 1
}

fatal() {
    echo "$*" >&2
    exit 1
}

cleanup() {
    echo "$TMPDIR"
#    rm -rf "$TMPDIR"
}

swap_bytes () {
  echo "${1:6:2}${1:4:2}${1:2:2}${1:0:2}"
}

while [ $# -gt 0 ] ; do
    case "$1" in
	--output|-o)
	    OUTPUT_FILE="$2"
	    shift
	    ;;
	--source|-s)
	    SOURCE_DIR="$2"
	    shift
	    ;;
	--key|-k)
	    KEY_FILE="$2"
	    shift
	    ;;
	--)
	    shift
	    break
	    ;;
	-h|--help)
	    usage
	    ;;
	-*)
	    usage
	    ;;
	*)
	    break
	    ;;
    esac
    shift
done

[ $# -eq 0 ] || usage
[ "$OUTPUT_FILE" ] || usage
[ "$SOURCE_DIR" ] || usage
[ "$KEY_FILE" ] || usage

TMPDIR="$(mktemp -d)"
trap cleanup EXIT

openssl rsa -in "$KEY_FILE" -pubout -outform der 2>/dev/null | \
    openssl dgst -sha256 -binary -out "$TMPDIR/extension.id"
truncate -s 16 "$TMPDIR/extension.id"

(cd "$SOURCE_DIR" && zip -qr -9 -X "$TMPDIR/extension.zip" .)

(
    printf "CRX3 SignedData"
    echo "00 12 00 00 00 0A 10" | xxd -r -p
    cat "$TMPDIR/extension.id" "$TMPDIR/extension.zip"
) | openssl dgst -sha256 -binary -sign "$KEY_FILE" > "$TMPDIR/extension.sig"

openssl rsa -pubout -outform DER < "$KEY_FILE" > "$TMPDIR/extension.pub" 2>/dev/null




crmagic_hex="43 72 32 34" # Cr24
version_hex="03 00 00 00" # 3
header_length="45 02 00 00"
header_chunk_1="12 AC 04 0A A6 02"
header_chunk_2="12 80 02"
header_chunk_3="82 F1 04 12 0A 10"
(
  echo "$crmagic_hex $version_hex $header_length $header_chunk_1" | xxd -r -p
  cat "$TMPDIR/extension.pub"
  echo "$header_chunk_2" | xxd -r -p
  cat "$TMPDIR/extension.sig"
  echo "$header_chunk_3" | xxd -r -p
  cat "$TMPDIR/extension.id" "$TMPDIR/extension.zip"
) > "$OUTPUT_FILE"


# 
# 
# crmagic_hex="4372 3234" # Cr24
# version_hex="0200 0000" # 2
# pub_len_hex=$(swap_bytes $(printf '%08x\n' $(ls -l "$TMPDIR/extension.pub" | awk '{print $5}')))
# sig_len_hex=$(swap_bytes $(printf '%08x\n' $(ls -l "$TMPDIR/extension.sig" | awk '{print $5}')))
# 
# (
#   echo "$crmagic_hex $version_hex $pub_len_hex $sig_len_hex" | xxd -r -p
#   cat "$TMPDIR/extension.pub" "$TMPDIR/extension.sig" "$TMPDIR/extension.zip"
# ) > "$OUTPUT_FILE"
# 
# 
